#include "Case.h"
#include "CaseBase.h"

#include <cassert>
#include <string>

using namespace std;

namespace cppcbr {

void AttributeSetCase::addAttribute(string name, AttributeValue* value) {
	attributes.insert(make_pair(name, value));
}

double AttributeSetCase::similarityTo(const SimilarityEvaluator* se, Case* c) const {
	AttributeSetCase* asc = dynamic_cast<AttributeSetCase*>(c);
	if (!asc) {
		throw string("AttributeSetCase objects can only measure similarity to other AttributeSetCase objects.");
	}

	// Query-biased similarity comparison
	double similarity = 0.0;
	for (attributes_type::const_iterator it = attributes.begin(); it != attributes.end(); ++it) {
		string name = it->first;
		attributes_type::const_iterator cit = asc->attributes.find(name);
		if (cit != asc->attributes.end()) {
			similarity += se->evaluateSimilarity(name, it->second, cit->second);
		}
	}
	return similarity;
}

AttributeValue* AttributeSetCase::getAttribute(string name) const {
	multimap<string, AttributeValue*>::const_iterator it = attributes.find(name);
	if (it != attributes.end()) {
		assert(it->first == name);
		return it->second;
	} else {
		return NULL;
	}
}

const AttributeSetCase::attributes_type& AttributeSetCase::getAttributes() const {
	return attributes;
}

}
