#ifndef CASE_H
#define CASE_H

#include <map>
#include <string>

namespace cppcbr {

class SimilarityEvaluator;

class Case {
public:
	Case() {}
	virtual ~Case() {}
	virtual double similarityTo(const SimilarityEvaluator* similarityEvaluator, Case* c) const = 0;
};

class AttributeValue {
public:
	AttributeValue() {}
	virtual ~AttributeValue() {}
};

template<typename T> class ClassAttributeValue: public AttributeValue {
public:
	ClassAttributeValue(T value): value(value) {}
	T getValue() const { return value; }
private:
	T value;
};

class AttributeSetCase: public Case {
public:
	typedef multimap<string, AttributeValue*> attributes_type;
	virtual void addAttribute(string name, AttributeValue* value);
	virtual AttributeValue* getAttribute(string name) const;
	virtual const attributes_type& getAttributes() const;
	virtual ~AttributeSetCase() {
		for (multimap<string, AttributeValue*>::iterator it = attributes.begin(); it != attributes.end(); ++it) {
			delete it->second;
		}
	}
	virtual double similarityTo(const SimilarityEvaluator* similarityEvaluator, Case* c) const;
private:
	attributes_type attributes;
};

}

#endif
