#ifndef CASERETRIEVER_H
#define CASERETRIEVER_H

#include <vector>
#include <memory>
#include <set>

namespace cppcbr {

class Case;
class CaseBase;
class SimilarityEvaluator;

template <typename C>
struct CaseRetriever {
	typedef C case_type;
	typedef vector<case_type*> retrieved_type;
	typedef set<case_type*> casebase_type;
	typedef set<pair<double, case_type*>, greater<pair<double, case_type*> > > similarities_type;
	vector<C*> operator()(const C* c, const casebase_type& cases, const SimilarityEvaluator* se) const {
		retrieved_type ret;
		similarities_type similarities;
		for (typename casebase_type::const_iterator it = cases.begin(); it != cases.end(); ++it) {
			double similarity = c->similarityTo(se, *it);
			similarities.insert(make_pair(similarity, *it));
		}
		for (typename similarities_type::const_iterator it = similarities.begin(); it != similarities.end(); ++it) {
			ret.push_back(it->second);
		}
		return ret;
	}
};

}

#endif
