#ifndef GOBOARD_H
#define GOBOARD_H

#include <iostream>

#include "common.h"

extern unsigned int hash_values[board_size][board_size][3];
extern int simuls;

namespace go {

ostream& operator<<(ostream& os, const position& p);

struct board_position {
	char board[board_size][board_size];
	board_position() {
		memset(board, 0, sizeof(board));
	}
};

struct group {
	int player;
	position stones_pos[board_size * board_size];
	int num_stones;
	position liberties_pos[board_size * board_size];
	bool liberties_map[board_size][board_size];
	int num_liberties;
	group(int player): player(player), num_stones(0), num_liberties(0) {
// 		memset(stones_pos, 0, sizeof(stones_pos));
// 		memset(liberties_pos, 0, sizeof(liberties_pos));
		memset(liberties_map, 0, sizeof(liberties_map));
	}
	void add_liberty(const position& p) {
		if (liberties_map[p.y][p.x]) return;
		liberties_map[p.y][p.x] = true;
		liberties_pos[num_liberties++] = p;
	}
	void add_stone(const position& p) {
		stones_pos[num_stones++] = p;
	}
};

struct move_status {
	bool legal;
	bool good;
	bool selfatari;
	bool finalendgame;
	int captures;
};

struct GoBoard;

struct GoBoard {
private:
	bool game_over;
	position last_play;
	vector<group> groups;
	set<unsigned int> visited_boards;
	bool some_group_in_atari;
protected:
	double get_board_tile_score() const;
	double get_board_estimate_tile_score() const;
	double eval_randomed_board();
	void turbo_rebuild_groups();
public:
	board_position bp;
	int group_idx[board_size][board_size];
	int player;
	int hash_code;
	int black_captures;
	int white_captures;
	int num_plays;
	double komi;

	GoBoard(): game_over(false), last_play(-3, -3), some_group_in_atari(false), player(1), hash_code(0), black_captures(0), white_captures(0), num_plays(0), komi(6.5) {
		groups.reserve(100);
		memset(group_idx, 0, sizeof(group_idx));
	}

	int getTile(const position& p) const { return bp.board[p.y][p.x]; }

	void setTile(const position& p, int v) {
		hash_code ^= hash_values[p.y][p.x][getTile(p)];
		bp.board[p.y][p.x] = v;
		hash_code ^= hash_values[p.y][p.x][v];
	}

	bool is_legal_hash(unsigned int hash) const { return !visited_boards.count(hash); }

	void build_groups();

	void pass() {
		assert(!game_over);
		int opponent = 3 - player;
		player = opponent;
		if (last_play.is_pass()) {
			game_over = true;
		}
		last_play = position(-1, -1);
	}

	void play(const position& pos);

	bool is_game_over() const { return game_over; }

	position get_last_play() const { return last_play; }

	bool isSingleStone(const position& p) const {
		vector<position> nbt = getNeighbourTiles(p);
		int player = getTile(p);
		for (unsigned int i = 0; i < nbt.size(); ++i) {
			if (getTile(nbt[i]) == player) return false;
		}
		return true;
	}

	vector<position> getNeighbourTiles(const position& p) const {
		vector<position> nbt;
		if (p.x > 0) nbt.push_back(position(p.x - 1, p.y));
		if (p.x < int(board_size - 1)) nbt.push_back(position(p.x + 1, p.y));
		if (p.y > 0) nbt.push_back(position(p.x, p.y - 1));
		if (p.y < int(board_size - 1)) nbt.push_back(position(p.x, p.y + 1));
		return nbt;
	}

	vector<position> getInterestingPlays() const;

	bool evalBoardMove(const position& pos, int purpose) const;

	int isCapturingMove(const position& pos, int player) const;

	bool isLegalPlay(const position& pos) const;

	double get_score() const {
		return get_board_tile_score() + black_captures - white_captures - komi;
	}

	double estimate_score() const {
		return get_board_estimate_tile_score() + black_captures - white_captures - komi;
	}

	void print() const;

	bool isSelfAtari(const position& pos, int player) const;

	double turbo_play();

protected:
	bool isLegalMove(const position& p, int player) const;
	bool isLegalHashPlay(const position& p) const;
	bool isConnectSplitMove(const position& p, int player) const;
public:
	bool isInterestingMove(const position& p, int player) const;
	bool isFinalEndgameMove(const position& p, int player) const;
	position get_funky_move() const;
};

}

#endif
