#include "CaseRetriever.h"
#include "CaseBase.h"
#include "Case.h"

#include <iostream>
#include <vector>
#include <string>
#include <sstream>
#include <typeinfo>

using namespace std;
using namespace cppcbr;

template<typename C> string tostring(C value) {
	stringstream ss;
	ss << value;
	return ss.str();
}

template<typename C> C getAttribute(const AttributeSetCase* c, const string& name) {
	AttributeValue* av = c->getAttribute(name);
	if (!av) {
		throw string("Attribute \"") + name + "\" does not exist for case " + tostring(c) + ".";
	}
	ClassAttributeValue<C>* cav = dynamic_cast<ClassAttributeValue<C>*>(av);
	if (!cav) {
		throw string("Attribute \"") + name + "\" is not of type " + typeid(C).name() + ".";
	}
	return cav->getValue();
}

template<typename T> double equality(const T& a, const T& b) {
	if (a == b) {
		return 1.0;
	} else {
		return 0.0;
	}
}

template<typename T> double difference(const T& a, const T& b) {
	double diff = a - b;
	if (diff < 0) diff = -diff;
	return 1 / (1 + diff);
}

string value_to_string(AttributeValue* value) {
	ClassAttributeValue<int>* iv = dynamic_cast<ClassAttributeValue<int>*>(value);
	if (iv) return tostring(iv->getValue());
	ClassAttributeValue<string>* sv = dynamic_cast<ClassAttributeValue<string>*>(value);
	if (sv) return tostring(sv->getValue());
	throw string("Unknown value type!");
}

void print_case(string title, AttributeSetCase* c) {
	cout << title << " (" << c << "):" << endl;
	const AttributeSetCase::attributes_type& attributes = c->getAttributes();
	for (AttributeSetCase::attributes_type::const_iterator it = attributes.begin(); it != attributes.end(); ++it) {
		cout << "   " << it->first << ": " << value_to_string(it->second) << endl;
	}
	cout << endl;
}

void test() {
	SimpleCaseBase cb;
	cb.addSimilarityMeasure("name", new ValueSimilarity<string>(equality<string>));
	cb.addSimilarityMeasure("sex", new ValueSimilarity<string>(equality<string>));
	cb.addSimilarityMeasure("age", new ValueSimilarity<int>(difference<int>));
	for (int i = 0; i < 10; ++i) {
		SimpleCaseBase::case_type* c = new SimpleCaseBase::case_type();
		c->addAttribute("name", new ClassAttributeValue<string>("foo" + tostring<int>(i)));
		c->addAttribute("sex", new ClassAttributeValue<string>((i%2)?"male":"female"));
		c->addAttribute("age", new ClassAttributeValue<int>((i * 733) % 90 + 10));
// 		cout << i << ": " << c << endl;
		cb.insertCase(c);
		print_case("Added case", c);
	}

	SimpleCaseBase::case_type* testCase = new SimpleCaseBase::case_type();
	testCase->addAttribute("name", new ClassAttributeValue<string>("test case"));
	testCase->addAttribute("age", new ClassAttributeValue<int>(90));
	testCase->addAttribute("sex", new ClassAttributeValue<string>("male"));

	print_case("Query case", testCase);

	SimpleCaseBase::retrieved_type similarCases = cb.retrieveSimilarCases(testCase);
	int case_num = 0;
	for (SimpleCaseBase::retrieved_type::const_iterator it = similarCases.begin(); it != similarCases.end(); ++it) {
		if (++case_num > 5) break;
		print_case("Similar case #" + tostring(case_num), *it);
	}
}
