#include <iostream>
#include <string>
#include <sstream>
#include <vector>

#include "UCT.h"

using namespace std;
using namespace go;

string prune_input_string(string s) {
	string output = "";
	for (unsigned int i = 0; i < s.size(); ++i) {
		char c = s[i];
		if (c == '\r') continue;
		if (c == '\t') c = ' ';
		if (c == '#') break;
		output += c;
	}
	return output;
}

struct gtp_command {
	int id;
	string cmd;
	vector<string> args;
};

gtp_command read_command() {
	string str;
	do {
		getline(cin, str);
		str = prune_input_string(str);
	} while (str == "");
	stringstream ss;
	ss << str;
	int id = -1;
	if (!(ss >> id)) ss.clear();
	string cmd;
	ss >> cmd;
	vector<string> args;
	while (ss >> str) {
		args.push_back(str);
	}
	cerr << "id[" << id << "] str[" << cmd << "]";
	for (unsigned int i = 0; i < args.size(); ++i) {
		cerr << " arg" << i << "[" << args[i] << "]";
	}
	cerr << endl;
	gtp_command ret = { id, cmd, args };
	return ret;
}

position vertex_to_position(string vertex) {
	char alpha = vertex[0];
	if (alpha >= 'A' && alpha <= 'Z') alpha = alpha - 'A' + 'a';
	int digit = vertex[1] - '1';
	if (alpha >= 'i') alpha -= 1;
	return position(alpha - 'a', digit);
}

string position_to_vertex(position p) {
	stringstream ss;
	char alpha = 'A' + p.x;
	if (alpha >= 'I') alpha++;
	int digit = p.y + 1;
	ss << alpha << digit;
	return ss.str();
}

void run_gtp_engine() {
	GoBoard<9> board;
	while (true) {
		gtp_command cmd = read_command();
		bool command_ok = true;
		string error_msg = "error interpreting command: " + cmd.cmd;
		string response = "";
		if (cmd.cmd == "protocol_version") {
			response = "2";
		} else if (cmd.cmd == "name") {
			response = "CBRGo";
		} else if (cmd.cmd == "version") {
			response = "0.01";
		} else if (cmd.cmd == "known_command") {
			response = "false";
		} else if (cmd.cmd == "list_commands") {
			response = "play\n";
		} else if (cmd.cmd == "quit") {
			return;
		} else if (cmd.cmd == "boardsize") {
			if (cmd.args[0] != "9" && cmd.args[0] != "19") {
				command_ok = false;
				error_msg = "unsupported board size";
			}
		} else if (cmd.cmd == "clear_board") {
		} else if (cmd.cmd == "komi") {
		} else if (cmd.cmd == "play") {
			position p = vertex_to_position(cmd.args[1]);
			board.play(p);
		} else if (cmd.cmd == "genmove") {
			UCT<9> uct(board);
			for (int i = 0; i < 10000; ++i) {
				uct.evaluateBoard();
			}
			position p = uct.get_best_move();
			board.play(p);
			response = position_to_vertex(p);
		}
		if (command_ok) {
			cout << "=";
			if (cmd.id > -1) cout << cmd.id;
			cout << " " << response;
			cout << endl << endl;
		} else {
			cout << "?";
			if (cmd.id > -1) cout << cmd.id;
			cout << " " << error_msg;
			cout << endl << endl;
		}
	}
}
