#include <cstdlib>
#include <iostream>

#include "common.h"

#include "UCT.h"
#include "extra.h"
#include "sgf.h"

#include "influence.h"

#include <iostream>
#include <iomanip>
#include <fstream>
#include <bitset>

bool using_cbr = false;

int simuls;

vector<game_record> influence_map;

unsigned int hash_values[board_size][board_size][3];

void init_hash() {
	for (int y = 0; y < board_size; ++y) {
		for (int x = 0; x < board_size; ++x) {
			hash_values[y][x][0] = rand();
			hash_values[y][x][1] = rand();
			hash_values[y][x][2] = rand();
		}
	}
}

position get_computer_ai_move(const GoBoard& board, int simuls) {
// 	int simuls = 1000;
	UCT uct(board);
	cout << "eval: ";
	for (int i = 0; i < simuls; ++i) {
		if ((i % (simuls / 20) == 0) || (i % 10000 == 0)) {
			cout << double(i)/simuls << "  " << flush;
// 			print_node_tree(uct.getRootNode());
		}
// 		cout << "  evi[" << i << "]" << endl;
		uct.evaluateBoard();
	}
	cout << endl;
// 	cout << simuls << " simulations done." << endl;
// 	cout << endl << endl;
	uct.print_node_tree();
// 	cout << "Nodes: " << uct.nodes.size() << endl;
	position p = uct.get_best_move();
	return p;
}

class Player {
public:
	virtual position play(const GoBoard& board) = 0;
	virtual ~Player() {}
};

class ComputerPlayer: public Player {
	int simuls;
public:
	ComputerPlayer(int simuls): simuls(simuls) {}
	virtual position play(const GoBoard& board) {
		using_cbr = false;
		position p = get_computer_ai_move(board, simuls);
		return p;
	}
};

class CBRComputerPlayer: public Player {
	int simuls;
public:
	CBRComputerPlayer(int simuls): simuls(simuls) {
		assert(using_cbr == true);
	}
	virtual position play(const GoBoard& board) {
		using_cbr = true;
		position p = get_computer_ai_move(board, simuls);
		using_cbr = false;
		return p;
	}
};

class HumanPlayer: public Player {
	virtual position play(const GoBoard& board) {
		position p = get_player_input_move(board);
		return p;
	}
};

position str_to_pos(string s) {
	for (int y = 0; y < board_size; ++y) {
		for (int x = 0; x < board_size; ++x) {
			if (s == pos_to_str(position(x, y))) return position(x, y);
		}
	}
	throw string("Position ") + s + " not recognized";
}

int main() {
	using_cbr = true;
	if (board_size != 19) using_cbr = false;
	init_hash();
	srand(0);
	GoBoard amateur_board;
	{
// 		vector<GoBoard> amateur_game = load_sgf_file("amateur-game.sgf");
// 		amateur_board = amateur_game[140];
// 		amateur_board = amateur_game[700];
	}
// 	vector<GoBoard> pro_game = load_sgf_file("games/sgf/Honinbo.Shusaku-Ota.Yuzo-1853-05-15.sgf");

	if (using_cbr) {
		assert(board_size == 19);
		cout << "Processing SGF files..." << endl;
		vector<string> sgf_files = read_file_lines("games/sgf-files");
		for (unsigned int i = 0; i < sgf_files.size(); ++i) {
			add_sgf_file("games/sgf/" + sgf_files[i], influence_map);
			if (i % 100 == 0) cout << i << " / " << sgf_files.size() << endl;
		}
		cout << "Number of influence maps: " << influence_map.size() << endl;
	}

// 	return 0;

	try {
		GoBoard board;
// 		board = pro_game[30];
// 		board = amateur_board;
// 		bool autoplay = false;
// 		auto_ptr<Player> black(new ComputerPlayer(10000));
// 		auto_ptr<Player> white(new ComputerPlayer(1000));
// 		auto_ptr<Player> white(new HumanPlayer());
		auto_ptr<Player> black(new HumanPlayer());
// 		auto_ptr<Player> black(new CBRComputerPlayer(5000));
		auto_ptr<Player> white(new CBRComputerPlayer(2000));
		vector<position> plays;

		while (!board.is_game_over()) {
			srand(0);
// 			cout << endl;
			for (unsigned int i = 0; i < plays.size(); ++i) {
// 				cout << "  board.play(str_to_pos(\"" << plays[i] << "\"));" << endl;
// 				cout << plays[i] << ",";
			}
// 			cout << endl;
			board.print();
// 			if (plays.size() >= 10) break;
			bool black_turn = (board.player == 1);
			if (black_turn) {
				position p = black->play(board);
				if (p.is_resign()) {
					cout << "Black resigns." << endl;
					break;
				}
				board.play(p);
				plays.push_back(p);
			} else {
				position p = white->play(board);
				if (p.is_resign()) {
					cout << "White resigns." << endl;
					break;
				}
				board.play(p);
				plays.push_back(p);
			}
// 			board.print(); break;
		}
		cout << "Done." << endl;
		return 0;
	} catch (const string& msg) {
		cerr << "Error thrown: " << msg << endl;
		return 1;
	} catch (...) {
		cerr << "Unknown error thrown." << endl;
		return 1;
	}
}
