#include "common.h"
#include "GoBoard.h"

#include <vector>
#include <string>
#include <iostream>
#include <fstream>

string read_file_contents(string filename) {
	ifstream ifs(filename.c_str());
	if (!ifs) return "";
	string str;
	string contents;
	while (getline(ifs, str)) {
		contents += str;
	}
// 	cout << contents << endl;
	return contents;
}

vector<string> read_file_lines(string filename) {
	vector<string> ret;
	ifstream ifs(filename.c_str());
	if (!ifs) return ret;
	string str;
	string contents;
	while (getline(ifs, str)) {
		ret.push_back(str);
	}
	return ret;
}

namespace go {

vector<GoBoard> parse_sgf(string contents) {
	assert(board_size == 19);
	vector<GoBoard> boards;
	GoBoard board;
	boards.push_back(board);
	int num_node = 0;
	for (unsigned int i = 0; i < contents.length(); ++i) {
		if (contents[i] == '(') {
			if (i == 0) {
			} else {
// 				cout << "unhandled multi-branch file" << endl;
				return vector<GoBoard >();
			}
		} else if (contents[i] == ')') {
		} else if (contents[i] == ';') {
			num_node++;
		} else {
			string cmd;
			for (; i < contents.length() && contents[i] != '['; ++i) {
				cmd += contents[i];
			}
			string param;
			for (++i; i < contents.length() && (contents[i] != ']' || contents[i-1] == '\\'); ++i) {
				param += contents[i];
			}
			if (num_node < 2) {
			} else if (cmd == "W") {
				if (board.player == 2) {
					position p(-1, -1);
					if (param.length() == 2) {
						int x = param[0] - 'a';
						int y = (board_size - 1) - (param[1] - 'a');
		// 				cout << "white[" << param << "]" << endl;
		// 				cout << "white(" << x << ", " << y << ")" << endl;
						p = position(x, y);
					}
					board.play(p);
					boards.push_back(board);
				} else {
// 					cout << "unhandled white double-move" << endl;
					return vector<GoBoard >();
				}
			} else if (cmd == "B") {
				if (board.player == 1) {
					position p(-1, -1);
					if (param.length() == 2) {
						int x = param[0] - 'a';
						int y = (board_size - 1) - (param[1] - 'a');
		// 				cout << "black[" << param << "]" << endl;
		// 				cout << "black(" << x << ", " << y << ")" << endl;
						p = position(x, y);
					}
					board.play(p);
					boards.push_back(board);
				} else {
// 					cout << "unhandled black double-move" << endl;
					return vector<GoBoard >();
				}
// 			} else if (cmd == "C") {
// 				// comment, ignored
			} else if (cmd == "BL") {
				// time remaining, ignored
			} else if (cmd == "WL") {
				// time remaining, ignored
			} else if (cmd == "OB") {
				// stones remaining, ignored
			} else if (cmd == "OW") {
				// stones remaining, ignored
			} else if (cmd == "TB") {
				// score, ignored
			} else if (cmd == "TW") {
				// score, ignored
			} else {
// 				cout << "unhandled command '" << cmd << "' / '" << param << "'" << endl;
				return vector<GoBoard >();
			}
// 			board.print();
		}
		if (boards.size() > 40) break;
	}
// 	boards.back().print();
	return boards;
}

vector<GoBoard> load_sgf_file(string filename) {
	string contents = read_file_contents(filename);
	if (contents == "") return vector<GoBoard>();
	return parse_sgf(contents);
}

}
