#include "common.h"
#include "tactics.h"
#include "influence.h"

namespace go {

vector<stone_group> get_groups(const GoBoard board) {
// 	print_board_influence(board, 0.1);
	board_influence influence = get_board_influence(board);

	int group[board_size][board_size];
	memset(group, 0, sizeof(group));

	for (int y = board_size - 1; y >= 0; --y) {
		for (int x = 0; x < board_size; ++x) {
			position p(x, y);
			double min_score = 2;
			if (x < 2 || x >= board_size - 2) min_score /= 2;
			if (y < 2 || y >= board_size - 2) min_score /= 2;
			double sum_white = influence.get(2, p) - influence.get(1, p) * 2;
			double sum_black = influence.get(1, p) - influence.get(2, p) * 2;
			if (sum_black >= min_score) group[y][x] = -1;
			else if (sum_white >= min_score) group[y][x] = -2;
		}
	}

	map<int, int> map_group_players;

	int group_num = 1;
	for (int y = board_size - 1; y >= 0; --y) {
		for (int x = 0; x < board_size; ++x) {
			int g = group[y][x];
			if (g < 0) {
				int offx[] = { -1, +1, 0, 0 };
				int offy[] = { 0, 0, -1, +1 };
				deque<position> q;
				q.push_back(position(x, y));
				while (q.size()) {
					position p = q.front();
					q.pop_front();
					group[p.y][p.x] = group_num;
					map_group_players[group_num] = -g;
					for (int i = 0; i < 4; ++i) {
						int x = p.x + offx[i];
						int y = p.y + offy[i];
						if (x < 0 || x >= board_size) continue;
						if (y < 0 || y >= board_size) continue;
						if (group[y][x] == g) q.push_back(position(x, y));
					}
				}
				group_num++;
			}
		}
	}

	vector<stone_group> stone_groups(group_num - 1);

	vector<set<int> > bordering(group_num - 1);

	for (unsigned int i = 0; i < stone_groups.size(); ++i) {
		stone_groups[i].player = map_group_players[i + 1];
	}

	for (int y = board_size - 1; y >= 0; --y) {
		for (int x = 0; x < board_size; ++x) {
			position p(x, y);
			int g = group[y][x];
			if (g) stone_groups[g - 1].group_area.push_back(p);
			if (board.getTile(p) && g > 1) {
				stone_groups[g - 1].stones.push_back(p);

				int offx[] = { -1, +1, 0, 0 };
				int offy[] = { 0, 0, -1, +1 };
				for (int i = 0; i < 4; ++i) {
					int x = p.x + offx[i];
					int y = p.y + offy[i];
					if (x < 0 || x >= board_size) continue;
					if (y < 0 || y >= board_size) continue;
					if (group[y][x] && group[y][x] != g) {
						bordering[g - 1].insert(group[y][x] - 1);
						bordering[group[y][x] - 1].insert(g - 1);
					}
				}
			}
		}
	}

	for (unsigned int i = 0; i < bordering.size(); ++i) {
		for (set<int>::const_iterator it = bordering[i].begin(); it != bordering[i].end(); ++it) {
			stone_groups[i].bordering_groups.push_back(*it);
		}
	}

	return stone_groups;
}

void get_group_status(const GoBoard& board, const stone_group& g) {
	GoBoard new_board;
	set<position> area;

	// Fill rest of board with opponents stones, riddled with 1-space eyes
	for (int y = board_size - 1; y >= 0; --y) {
		for (int x = 0; x < board_size; ++x) {
			new_board.setTile(position(x, y), 3 - g.player);
		}
	}
	for (unsigned int i = 0; i < g.group_area.size(); ++i) {
		area.insert(g.group_area[i]);
		new_board.setTile(g.group_area[i], 0);
	}
	for (unsigned int i = 0; i < g.stones.size(); ++i) {
		new_board.setTile(g.stones[i], g.player);
	}
	for (int y = board_size - 2; y >= 1; --y) {
		for (int x = 1; x < board_size - 1; ++x) {
			bool filled = true;
			for (int dy = -1; dy <= 1; ++dy) {
				for (int dx = -1; dx <= 1; ++dx) {
					if (new_board.getTile(position(x + dx, y + dy)) != 3 - g.player) filled = false;
				}
			}
			if (filled) {
				new_board.setTile(position(x, y), 0);
			}
		}
	}

	new_board.print();
}

double get_board_score_estimate(const GoBoard& board) {
	board_influence influence = get_board_influence(board);

	vector<stone_group> groups = get_groups(board);

	int score_board[board_size][board_size];
	memset(score_board, 0, sizeof(score_board));

	for (unsigned int i = 0; i < groups.size(); ++i) {
		const stone_group& g = groups[i];
		int gsize = g.group_area.size();
		// Only count groups that are likely to live
		if (gsize >= 3) {
			for (unsigned int j = 0; j < g.group_area.size(); ++j) {
				const position& p = g.group_area[j];
				score_board[p.y][p.x] = g.player;
			}
		}
	}

	double score = 0;
	for (int y = 0; y < board_size; ++y) {
		for (int x = 0; x < board_size; ++x) {
			if (score_board[y][x] == 1) ++score;
			if (score_board[y][x] == 2) --score;
		}
	}

	board.print();
	print_board_influence(board);
	for (int y = board_size - 1; y >= 0; --y) {
		for (int x = 0; x < board_size; ++x) {
			cout << " " << score_board[y][x];
		}
		cout << endl;
	}
	cout << "score est: " << score << endl;

	return score;
}

}
